package tech.mercantec.easyeat.ui.dishes

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.os.Bundle
import android.text.Html
import android.util.Log
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import android.widget.Button
import android.widget.LinearLayout
import android.widget.Toast
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.getRecipeDetails
import kotlin.concurrent.thread
import android.widget.EditText
import android.widget.ImageButton
import android.widget.TextView
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.core.widget.doAfterTextChanged
import tech.mercantec.easyeat.helpers.AddRecipeToShoppingList
import tech.mercantec.easyeat.models.Ingredient
import tech.mercantec.easyeat.models.Recipe

class DishDetailsActivity : AppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_dish_details)

        val dishId = intent.getIntExtra("dish_id", -1)
        if (dishId == -1) {
            Toast.makeText(this, "No dish ID provided", Toast.LENGTH_SHORT).show()
            finish()
            return
        }

        findViewById<Button>(R.id.addDishToShoppingList).setOnClickListener {
            val progressDialog = ProgressDialog(this)
            progressDialog.setMessage("Loading...")
            progressDialog.show()
            thread {
                try {
                    val multiplierText = findViewById<EditText>(R.id.ingredient_multiplier).text.toString()
                    AddRecipeToShoppingList(this, dishId, multiplierText)
                } catch (e: ApiRequestException) {
                    runOnUiThread {
                        Toast.makeText(this, e.message, Toast.LENGTH_LONG).show()
                    }
                    return@thread
                } finally {
                    runOnUiThread {
                        progressDialog.dismiss()
                    }
                }

                runOnUiThread {
                    Toast.makeText(this, "Recipe ingredients added to shopping", Toast.LENGTH_LONG).show()
                    setResult(Activity.RESULT_OK)
                    finish()
                }
            }
        }

        // Load the recipe
        loadRecipe(dishId)
    }


    private fun loadRecipe(dishId: Int) {
        val ingredientsContainer = findViewById<LinearLayout>(R.id.ingredients)
        val multiplierEditText = findViewById<EditText>(R.id.ingredient_multiplier)
        val instructionsLayout = findViewById<LinearLayout>(R.id.instructions)

        thread {
            val recipe: Recipe
            try {
                recipe = getRecipeDetails(this, dishId)
            } catch (e: ApiRequestException) {
                runOnUiThread {
                    Toast.makeText(this, e.message, Toast.LENGTH_LONG).show()
                }
                return@thread
            }

            fun displayIngredients(ingredients: List<Ingredient>, multiplier: Int) {
                runOnUiThread {
                    ingredientsContainer.removeAllViews()

                    for (ingredient in ingredients) {
                        val row = TextView(this)
                        val amount = ingredient.amount?.times(multiplier)
                            ?.toBigDecimal()?.stripTrailingZeros()?.toPlainString()
                        val amountStr = if (amount != null) ": $amount" else ""

                        row.text = Html.fromHtml(
                            "&#8226; ${ingredient.name}$amountStr ${ingredient.unit ?: ""}",
                            Html.FROM_HTML_MODE_LEGACY
                        )
                        row.textSize = 18f
                        row.setPadding(0, 8, 0, 8)
                        ingredientsContainer.addView(row)
                    }
                }
            }

            runOnUiThread {
                findViewById<TextView>(R.id.dishDetailName).text = recipe.name
                findViewById<TextView>(R.id.dishDetailDescription).text = recipe.description
                instructionsLayout.removeAllViews()

                recipe.directions.forEachIndexed { index, direction ->
                    val textView = TextView(this).apply {
                        text = "${index + 1}. $direction"
                        textSize = 18f
                        setPadding(0, 8, 0, 8)
                    }
                    instructionsLayout.addView(textView)
                }

                var multiplier = 1
                displayIngredients(recipe.ingredients, multiplier)

                multiplierEditText.doAfterTextChanged {
                    multiplier = it.toString().toIntOrNull() ?: 1
                    displayIngredients(recipe.ingredients, multiplier)
                }

                findViewById<ImageButton>(R.id.increment_multiplier).setOnClickListener {
                    multiplier++
                    multiplierEditText.setText(multiplier.toString(), TextView.BufferType.EDITABLE)
                    displayIngredients(recipe.ingredients, multiplier)
                }

                findViewById<ImageButton>(R.id.decrement_multiplier).setOnClickListener {
                    if (multiplier > 1) {
                        multiplier--
                        multiplierEditText.setText(multiplier.toString(), TextView.BufferType.EDITABLE)
                        displayIngredients(recipe.ingredients, multiplier)
                    }
                }
            }
        }
    }

}