package tech.mercantec.easyeat.ui.dishes

import android.app.Activity
import android.app.ProgressDialog
import android.os.Bundle
import android.view.View
import android.widget.Button
import android.widget.CheckBox
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.createRecipe
import tech.mercantec.easyeat.helpers.generateRecipeWithAI
import java.util.Locale
import kotlin.concurrent.thread

class GenerateRecipeActivity : AppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_generate_recipe)

        val allergiesInput = findViewById<EditText>(R.id.allergies)

        findViewById<CheckBox>(R.id.has_allergies).setOnCheckedChangeListener { _, checked ->
            allergiesInput.visibility = if (checked) View.VISIBLE else View.GONE
        }

        findViewById<Button>(R.id.generate).setOnClickListener {
            val name = findViewById<EditText>(R.id.dish_title).text.toString()
            val allergies =
                if (findViewById<CheckBox>(R.id.has_allergies).isChecked)
                    allergiesInput.text.split(Regex(",\\s*"))
                else
                    listOf()

            val progressDialog = ProgressDialog(this)
            progressDialog.setMessage("Generating...")
            progressDialog.show()

            thread {
                try {
                    val recipe = generateRecipeWithAI(this, name, Locale.getDefault().displayLanguage, allergies)

                    runOnUiThread {
                        progressDialog.setMessage("Saving...")
                    }

                    createRecipe(this, recipe)

                    setResult(Activity.RESULT_OK)
                    finish()
                } catch (e: ApiRequestException) {
                    runOnUiThread {
                        Toast.makeText(this, e.message, Toast.LENGTH_LONG).show()
                    }
                } finally {
                    runOnUiThread {
                        progressDialog.hide()
                    }
                }
            }
        }
    }
}
